@@ -14,14 +14,13 @@ PKI_DIR = getattr(settings, 'PKI_DIR', os.path.join(PKI_APP_DIR, 'PKI'))
 PKI_OPENSSL_BIN = getattr(settings, 'PKI_OPENSSL_BIN', '/usr/bin/openssl')
 
 # path to generated openssl.conf
-PKI_OPENSSL_CONF = getattr(settings, 'PKI_OPENSSL_CONF',
-                           os.path.join(PKI_DIR, 'openssl.conf'))
+PKI_OPENSSL_CONF = getattr(settings, 'PKI_OPENSSL_CONF', os.path.join(PKI_DIR, 'openssl.conf'))
 
 # template name for openssl.conf
 PKI_OPENSSL_TEMPLATE = getattr(settings, 'PKI_OPENSSL_TEMPLATE', 'pki/openssl.conf.in')
 
 # jquery url (defaults to pki/jquery-1.3.2.min.js)
-JQUERY_URL = getattr(settings, 'JQUERY_URL', 'pki/jquery-1.3.2.min.js')
+JQUERY_URL = getattr(settings, 'JQUERY_URL', 'pki/jquery-1.4.2.min.js')
 
 # logging (TODO: syslog, handlers and formatters)
 PKI_LOG = getattr(settings, 'PKI_LOG', os.path.join(PKI_DIR, 'pki.log'))
@@ -48,3 +47,6 @@ PKI_ENABLE_GRAPHVIZ = getattr(settings, 'PKI_ENABLE_GRAPHVIZ', False)
 # graphviz direction: From left to right (LR) or top down (TD)
 PKI_GRAPHVIZ_DIRECTION = getattr(settings, 'PKI_GRAPHVIZ_DIRECTION', 'LR')
 
+# enable email delivery: Certificates with defined email address can be sent via email
+PKI_ENABLE_EMAIL = getattr(settings, 'PKI_ENABLE_EMAIL', False)
+
